@@ -110,28 +110,46 @@ static unsigned int
 %  any of the following bits of information about the image by embedding
 %  the appropriate special characters:
 %
-%    %b   file size in bytes.
-%    %c   comment.
-%    %d   directory in which the image resides.
-%    %e   extension of the image file.
-%    %f   original filename of the image.
-%    %h   height of image.
-%    %i   filename of the image.
-%    %k   number of unique colors.
-%    %l   image label.
-%    %m   image file format.
-%    %n   number of images in a image sequence.
-%    %o   output image filename.
-%    %p   page number of the image.
-%    %q   image depth (8 or 16).
-%    %p   page number of the image.
-%    %q   image depth (8 or 16).
-%    %s   image scene number.
-%    %t   image filename without any extension.
-%    %u   a unique temporary filename.
-%    %w   image width.
-%    %x   x resolution of the image.
-%    %y   y resolution of the image.
+%    %b   file size
+%    %c   comment
+%    %d   directory
+%    %e   filename extension
+%    %f   filename
+%    %g   page dimensions and offsets
+%    %h   height
+%    %i   input filename
+%    %k   number of unique colors
+%    %l   label
+%    %m   magick
+%    %n   number of scenes
+%    %o   output filename
+%    %p   page number
+%    %q   image bit depth
+%    %r   image type description
+%    %s   scene number
+%    %t   top of filename
+%    %w   width
+%    %x   horizontal resolution
+%    %y   vertical resolution
+%    %A   transparency supported
+%    %C   compression type
+%    %D   GIF disposal method
+%    %G   Original width and height
+%    %H   page height
+%    %M   original filename specification
+%    %O   page offset (x,y)
+%    %P   page dimensions (width,height)
+%    %T   time delay (in centi-seconds)
+%    %U   resolution units
+%    %W   page width
+%    %X   page horizontal offset (x)
+%    %Y   page vertical offset (y)
+%    %@   trim bounding box
+%    %[a] named attribute 'a'
+%    %#   signature
+%    \n   newline
+%    \r   carriage return
+%    %%   % (literal)
 %
 %  The format of the AnnotateImage method is:
 %
@@ -194,7 +212,7 @@ MagickExport MagickPassFail AnnotateImage(Image *image,const DrawInfo *draw_info
   if (draw_info->text == (char *) NULL)
     return(MagickFail);
   if (*draw_info->text == '\0')
-    return(MagickFail);
+    return(MagickPass);
   text=TranslateText((ImageInfo *) NULL,image,draw_info->text);
   if (text == (char *) NULL)
     ThrowBinaryException3(ResourceLimitError,MemoryAllocationFailed,
@@ -219,7 +237,7 @@ MagickExport MagickPassFail AnnotateImage(Image *image,const DrawInfo *draw_info
   annotate=CloneDrawInfo((ImageInfo *) NULL,draw_info);
   clone_info=CloneDrawInfo((ImageInfo *) NULL,draw_info);
   matte=image->matte;
-  status=True;
+  status=MagickPass;
   for (i=0; textlist[i] != (char *) NULL; i++)
   {
     if (*textlist[i] == '\0')
@@ -1010,6 +1028,9 @@ static MagickPassFail RenderFreetype(Image *image,const DrawInfo *draw_info,
       0, 0
     };
 
+  MagickBool
+    active;
+
   magick_code_point_t
     *text;
 
@@ -1285,66 +1306,85 @@ static MagickPassFail RenderFreetype(Image *image,const DrawInfo *draw_info,
                 {
                   continue;
                 }
+              /*
+                Try to get whole span.  May fail.
+              */
               q=GetImagePixels(image,(long) ceil(point.x-0.5),
                 (long) ceil(point.y+y-0.5),bitmap->bitmap.width,1);
-              if (q == (PixelPacket *) NULL)
-                status=MagickFail;
-              if (status != MagickFail)
+              active=q != (PixelPacket *) NULL;
+              for (x=0; x < (long) bitmap->bitmap.width; x++, pc++)
                 {
-                  for (x=0; x < (long) bitmap->bitmap.width; x++, pc++)
+                  if (((long) ceil(point.x+x-0.5) < 0) ||
+                      ((unsigned long) ceil(point.x+x-0.5) >= image->columns))
                     {
-                      if (((long) ceil(point.x+x-0.5) < 0) ||
-                          ((unsigned long) ceil(point.x+x-0.5) >= image->columns))
-                        {
-                          q++;
-                          continue;
-                        }
-                      /* 8-bit gray-level pixmap */
-                      if (bitmap->bitmap.pixel_mode == ft_pixel_mode_grays)
-                        {
-                          if (draw_info->text_antialias)
-                            opacity=ScaleCharToQuantum(p[pc]);
-                          else
-                            opacity=(p[pc] < 127 ? OpaqueOpacity : TransparentOpacity);
-                        }
-                      /* 1-bit monochrome bitmap */
-                      else if (bitmap->bitmap.pixel_mode == ft_pixel_mode_mono)
-                        {
-                          opacity=((p[(x >> 3) + pcr] & (1 << (~x & 0x07))) ?
-                                   TransparentOpacity : OpaqueOpacity);
-                        }
+                      if (active)
+                        q++;
+                      continue;
+                    }
+                  /* 8-bit gray-level pixmap */
+                  if (bitmap->bitmap.pixel_mode == ft_pixel_mode_grays)
+                    {
+                      if (draw_info->text_antialias)
+                        opacity=ScaleCharToQuantum(p[pc]);
                       else
-                        {
-                          continue; /* ignore it? */
-                        }
-                      fill_color=draw_info->fill;
-                      if (pattern != (Image *) NULL)
-                        {
-                          if (AcquireOnePixelByReference
-                              (pattern,&fill_color,
-                               (long) (point.x+x-pattern->tile_info.x) % pattern->columns,
-                               (long) (point.y+y-pattern->tile_info.y) % pattern->rows,
-                               &image->exception) == MagickFail)
-                            {
-                              status=MagickFail;
-                            }
-                        }
-                      if (status != MagickFail)
+                        opacity=(p[pc] < 127 ? OpaqueOpacity : TransparentOpacity);
+                    }
+                  /* 1-bit monochrome bitmap */
+                  else if (bitmap->bitmap.pixel_mode == ft_pixel_mode_mono)
+                    {
+                      opacity=((p[(x >> 3) + pcr] & (1 << (~x & 0x07))) ?
+                               TransparentOpacity : OpaqueOpacity);
+                    }
+                  else
+                    {
+                      continue; /* ignore it? */
+                    }
+                  fill_color=draw_info->fill;
+                  if (pattern != (Image *) NULL)
+                    {
+                      if (AcquireOnePixelByReference
+                          (pattern,&fill_color,
+                           (long) (point.x+x-pattern->tile_info.x) % pattern->columns,
+                           (long) (point.y+y-pattern->tile_info.y) % pattern->rows,
+                           &image->exception) == MagickFail)
                         {
-                          opacity=((MaxRGB-opacity)*(MaxRGB-fill_color.opacity))/MaxRGB;
-                          AlphaCompositePixel(q,&fill_color,opacity,q,
-                                              image->matte ? q->opacity : OpaqueOpacity);
-                          (void) SyncImagePixels(image);
+                          status=MagickFail;
                         }
+                    }
+                  /*
+                    If not full span, then get one pixel.
+                  */
+                  if (!active)
+                    q=GetImagePixels(image,(long) ceil(point.x+x-0.5),
+                                     (long) ceil(point.y+y-0.5),1,1);
+                  if (q == (PixelPacket *) NULL)
+                    {
+                      continue;
+                    }
+                  opacity=((MaxRGB-opacity)*(MaxRGB-fill_color.opacity))/MaxRGB;
+                  AlphaCompositePixel(q,&fill_color,opacity,q,
+                                      image->matte ? q->opacity : OpaqueOpacity);
+                  if (!active)
+                    {
+                      /*
+                        Sync the one pixel
+                      */
+                      if (SyncImagePixels(image) != MagickPass)
+                        status=MagickFail;
+                    }
+                  else
+                    {
                       q++;
-                      if (status == MagickFail)
-                        break;
                     }
-                  if (SyncImagePixels(image) != MagickPass)
-                    status=MagickFail;
                   if (status == MagickFail)
                     break;
                 }
+              /*
+                Sync the full span
+              */
+              if (active)
+                if (SyncImagePixels(image) != MagickPass)
+                  status=MagickFail;
               if (status == MagickFail)
                 break;
             }
