@@ -1151,6 +1151,8 @@ MagickExport MagickPassFail ModulateImage(Image *image,const char *modulate)
       (void) ModulateImagePixels(NULL,&param,image,image->colormap,
                                  (IndexPacket *) NULL,image->colors,
                                  &image->exception);
+      MagickMonitorFormatted(image->colors,image->colors+1,&image->exception,
+                             progress_message,image->filename);
       status=SyncImage(image);
     }
   else
