@@ -68,7 +68,7 @@ static void AddTemporaryFileToList(const char *filename)
     if (info)
       {
         info->next=0;
-        (void) strcpy(info->filename,filename);
+        (void) strlcpy(info->filename,filename,sizeof(info->filename));
         if (!templist)
           templist=info;
         else
@@ -252,21 +252,27 @@ MagickExport int AcquireTemporaryFileDescriptor(char *filename)
       /*
         Use our own temporary filename generator if the temporary
         file directory is known.
+
+        In practice, this method is virtually always used.
       */
       char
-        tempname[MaxTextExtent];
+        tempname[16];
       
       int
         tries=0;
       
       for (tries=0; tries < 256; tries++)
         {
-          (void) strcpy(tempname,"gmXXXXXX");
+          (void) strlcpy(tempname,"gmXXXXXX",sizeof(tempname));
           ComposeTemporaryFileName(tempname);
-          (void) strcpy(filename,tempdir);
-          if (tempdir[strlen(tempdir)-1] != DirectorySeparator[0])
-            (void) strcat(filename,DirectorySeparator);
-          (void) strcat(filename,tempname);
+          if (strlcpy(filename,tempdir,MaxTextExtent) >= MaxTextExtent)
+            break;
+          if (filename[strlen(filename)-1] != DirectorySeparator[0])
+            if (strlcat(filename,DirectorySeparator,MaxTextExtent) >=
+                MaxTextExtent)
+              break;
+          if (strlcat(filename,tempname,MaxTextExtent) >= MaxTextExtent)
+            break;
           fd=open(filename,O_RDWR | O_CREAT | O_BINARY | O_EXCL, S_MODE);
           if (fd != -1)
             {
@@ -548,3 +554,38 @@ MagickExport void PurgeTemporaryFiles(void)
       MagickFreeMemory(liberate);
     }
 }
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
++   P u r g e T e m p o r a r y F i l e s A s y n c S a f e                   %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  PurgeTemporaryFiles reclaims all currently allocated temporary files,
+%  removing the files from the filesystem if they still exist.  This version
+%  is similar to PurgeTemporaryFiles() except that it intentionally does
+%  not release any memory (might use a mutex/semaphore) or invoke any
+%  functions which are not async-safe.  The only reason to call this function
+%  is something has gone wrong and the program needs to quit immediately.
+%
+%      void PurgeTemporaryFilesAsyncSafe(void)
+%
+*/
+MagickExport void PurgeTemporaryFilesAsyncSafe(void)
+{
+  const TempfileInfo
+    *member;
+
+  member=templist;
+  templist=0;
+  while(member)
+    {
+      (void) unlink(member->filename);
+      member=member->next;
+    }
+}
